@@ -117,6 +117,7 @@ static cptr r_info_blow_effect[] =
 	"EXP_20",
 	"EXP_40",
 	"EXP_80",
+	"HALLU",
 	NULL
 };
 
@@ -274,7 +275,7 @@ static cptr r_info_flags4[] =
 	"XXX5X4",
 	"XXX6X4",
 	"XXX7X4",
-	"XXX8X4"
+	"BOULDER"
 };
 
 /*
@@ -341,7 +342,7 @@ static cptr r_info_flags6[] =
 	"S_HI_DEMON",
 	"S_MONSTER",
 	"S_MONSTERS",
-	"S_ANT",
+	"S_ANIMAL",
 	"S_SPIDER",
 	"S_HOUND",
 	"S_HYDRA",
@@ -386,8 +387,8 @@ static cptr k_info_flags1[] =
 	"SLAY_GIANT",
 	"SLAY_DRAGON",
 	"KILL_DRAGON",
-	"XXX5",
-	"XXX6",
+	"KILL_DEMON",
+	"KILL_UNDEAD",
 	"BRAND_POIS",
 	"BRAND_ACID",
 	"BRAND_ELEC",
@@ -502,8 +503,8 @@ static cptr a_info_act[ACT_MAX] =
 	"SLEEP",
 	"LIGHTNING_BOLT",
 	"ELEC2",
-	"GENOCIDE",
-	"MASS_GENOCIDE",
+	"BANISHMENT",
+	"MASS_BANISHMENT",
 	"IDENTIFY",
 	"DRAIN_LIFE1",
 	"DRAIN_LIFE2",
@@ -525,7 +526,10 @@ static cptr a_info_act[ACT_MAX] =
 	"WOR",
 	"CONFUSE",
 	"PROBE",
-	"FIREBRAND"
+	"FIREBRAND",
+	"STARLIGHT",
+	"MANA_BOLT",
+	"BERSERKER"
 };
 
 
@@ -714,8 +718,7 @@ static u32b add_name(header *head, cptr buf)
  */
 errr parse_z_info(char *buf, header *head)
 {
-	/* Unused parameter */
-	(void)head;
+	maxima *z_info = head->info_ptr;
 
 	/* Hack - Verify 'M:x:' format */
 	if (buf[0] != 'M') return (PARSE_ERROR_UNDEFINED_DIRECTIVE);
@@ -845,6 +848,18 @@ errr parse_z_info(char *buf, header *head)
 		z_info->b_max = max;
 	}
 
+	/* Process 'L' for "Maximum flavor_info[] subindex" */
+	else if (buf[2] == 'L')
+	{
+		int max;
+
+		/* Scan for the value */
+		if (1 != sscanf(buf+4, "%d", &max)) return (PARSE_ERROR_GENERIC);
+
+		/* Save the value */
+		z_info->flavor_max = max;
+	}
+	
 	/* Process 'O' for "Maximum o_list[] index" */
 	else if (buf[2] == 'O')
 	{
@@ -857,7 +872,7 @@ errr parse_z_info(char *buf, header *head)
 		z_info->o_max = max;
 	}
 
-	/* Process 'M' for "Maximum m_list[] index" */
+	/* Process 'M' for "Maximum mon_list[] index" */
 	else if (buf[2] == 'M')
 	{
 		int max;
@@ -944,7 +959,7 @@ errr parse_v_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		v_ptr = &v_info[i];
+		v_ptr = (vault_type*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(v_ptr->name = add_name(head, s)))
@@ -982,6 +997,13 @@ errr parse_v_info(char *buf, header *head)
 		v_ptr->rat = rat;
 		v_ptr->hgt = hgt;
 		v_ptr->wid = wid;
+
+		/* Check for maximum vault sizes */
+		if ((v_ptr->typ == 7) && ((v_ptr->wid > 33) || (v_ptr->hgt > 22)))
+			return (PARSE_ERROR_VAULT_TOO_BIG);
+
+		if ((v_ptr->typ == 8) && ((v_ptr->wid > 66) || (v_ptr->hgt > 44)))
+			return (PARSE_ERROR_VAULT_TOO_BIG);
 	}
 	else
 	{
@@ -1036,7 +1058,7 @@ errr parse_f_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		f_ptr = &f_info[i];
+		f_ptr = (feature_type*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(f_ptr->name = add_name(head, s)))
@@ -1065,7 +1087,8 @@ errr parse_f_info(char *buf, header *head)
 	/* Process 'G' for "Graphics" (one line only) */
 	else if (buf[0] == 'G')
 	{
-		int tmp;
+		char d_char;
+		int d_attr;
 
 		/* There better be a current f_ptr */
 		if (!f_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
@@ -1075,15 +1098,30 @@ errr parse_f_info(char *buf, header *head)
 		if (!buf[3]) return (PARSE_ERROR_GENERIC);
 		if (!buf[4]) return (PARSE_ERROR_GENERIC);
 
-		/* Extract the attr */
-		tmp = color_char_to_attr(buf[4]);
+		/* Extract d_char */
+		d_char = buf[2];
+
+		/* If we have a longer string than expected ... */
+		if (buf[5])
+		{
+			/* Advance "buf" on by 4 */
+			buf += 4;
+
+			/* Extract the colour */
+			d_attr = color_text_to_attr(buf);
+		}
+		else
+		{
+			/* Extract the attr */
+			d_attr = color_char_to_attr(buf[4]);
+		}
 
 		/* Paranoia */
-		if (tmp < 0) return (PARSE_ERROR_GENERIC);
+		if (d_attr < 0) return (PARSE_ERROR_GENERIC);
 
 		/* Save the values */
-		f_ptr->d_attr = tmp;
-		f_ptr->d_char = buf[2];
+		f_ptr->d_attr = d_attr;
+		f_ptr->d_char = d_char;
 	}
 	else
 	{
@@ -1181,7 +1219,7 @@ errr parse_k_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		k_ptr = &k_info[i];
+		k_ptr = (object_kind*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(k_ptr->name = add_name(head, s)))
@@ -1191,8 +1229,8 @@ errr parse_k_info(char *buf, header *head)
 	/* Process 'G' for "Graphics" (one line only) */
 	else if (buf[0] == 'G')
 	{
-		char sym;
-		int tmp;
+		char d_char;
+		int d_attr;
 
 		/* There better be a current k_ptr */
 		if (!k_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
@@ -1202,18 +1240,30 @@ errr parse_k_info(char *buf, header *head)
 		if (!buf[3]) return (PARSE_ERROR_GENERIC);
 		if (!buf[4]) return (PARSE_ERROR_GENERIC);
 
-		/* Extract the char */
-		sym = buf[2];
+		/* Extract d_char */
+		d_char = buf[2];
 
-		/* Extract the attr */
-		tmp = color_char_to_attr(buf[4]);
+		/* If we have a longer string than expected ... */
+		if (buf[5])
+		{
+			/* Advance "buf" on by 4 */
+			buf += 4;
+
+			/* Extract the colour */
+			d_attr = color_text_to_attr(buf);
+		}
+		else
+		{
+			/* Extract the attr */
+			d_attr = color_char_to_attr(buf[4]);
+		}
 
 		/* Paranoia */
-		if (tmp < 0) return (PARSE_ERROR_GENERIC);
+		if (d_attr < 0) return (PARSE_ERROR_GENERIC);
 
 		/* Save the values */
-		k_ptr->d_attr = tmp;
-		k_ptr->d_char = sym;
+		k_ptr->d_attr = d_attr;
+		k_ptr->d_char = d_char;
 	}
 
 	/* Process 'I' for "Info" (one line only) */
@@ -1335,6 +1385,21 @@ errr parse_k_info(char *buf, header *head)
 			s = t;
 		}
 	}
+
+	/* Process 'D' for "Description" */
+	else if (buf[0] == 'D')
+	{
+		/* There better be a current k_ptr */
+		if (!k_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
+
+		/* Get the text */
+		s = buf+2;
+
+		/* Store the text */
+		if (!add_text(&(k_ptr->text), head, s))
+			return (PARSE_ERROR_OUT_OF_MEMORY);
+	}
+
 	else
 	{
 		/* Oops */
@@ -1435,7 +1500,7 @@ errr parse_a_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		a_ptr = &a_info[i];
+		a_ptr = (artifact_type*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(a_ptr->name = add_name(head, s)))
@@ -1551,7 +1616,7 @@ errr parse_a_info(char *buf, header *head)
 		if (!*s) return (PARSE_ERROR_GENERIC);
 
 		/* Get the activation */
-		grab_one_activation(a_ptr, buf + 2);
+		if (grab_one_activation(a_ptr, buf + 2)) return (PARSE_ERROR_GENERIC);
 
 		/* Scan for the values */
 		if (2 != sscanf(s, "%d:%d",
@@ -1639,7 +1704,7 @@ errr parse_e_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		e_ptr = &e_info[i];
+		e_ptr = (ego_item_type*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(e_ptr->name = add_name(head, s)))
@@ -1672,17 +1737,16 @@ errr parse_e_info(char *buf, header *head)
 	/* Process 'X' for "Xtra" (one line only) */
 	else if (buf[0] == 'X')
 	{
-		int slot, rating, xtra;
+		int rating, xtra;
 
 		/* There better be a current e_ptr */
 		if (!e_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
 
 		/* Scan for the values */
-		if (3 != sscanf(buf+2, "%d:%d:%d",
-			            &slot, &rating, &xtra)) return (PARSE_ERROR_GENERIC);
+		if (2 != sscanf(buf+2, "%d:%d", &rating, &xtra))
+			return (PARSE_ERROR_GENERIC);
 
 		/* Save the values */
-		e_ptr->slot = slot;
 		e_ptr->rating = rating;
 		e_ptr->xtra = xtra;
 	}
@@ -1704,11 +1768,11 @@ errr parse_e_info(char *buf, header *head)
 		e_ptr->min_sval[cur_t] = (byte)sval1;
 		e_ptr->max_sval[cur_t] = (byte)sval2;
 
-		/* increase counter for 'possible tval' index */
+		/* Increase counter for 'possible tval' index */
 		cur_t++;
 
-		/* only three T: lines allowed */
-		if (cur_t > 3) return (PARSE_ERROR_GENERIC);
+		/* Allow only a limited number of T: lines */
+		if (cur_t > EGO_TVALS_MAX) return (PARSE_ERROR_GENERIC);
 	}
 
 	/* Hack -- Process 'C' for "creation" */
@@ -1853,7 +1917,7 @@ errr parse_r_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		r_ptr = &r_info[i];
+		r_ptr = (monster_race*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(r_ptr->name = add_name(head, s)))
@@ -1877,8 +1941,8 @@ errr parse_r_info(char *buf, header *head)
 	/* Process 'G' for "Graphics" (one line only) */
 	else if (buf[0] == 'G')
 	{
-		char sym;
-		int tmp;
+		char d_char;
+		int d_attr;
 
 		/* There better be a current r_ptr */
 		if (!r_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
@@ -1888,18 +1952,30 @@ errr parse_r_info(char *buf, header *head)
 		if (!buf[3]) return (PARSE_ERROR_GENERIC);
 		if (!buf[4]) return (PARSE_ERROR_GENERIC);
 
-		/* Extract the char */
-		sym = buf[2];
+		/* Extract d_char */
+		d_char = buf[2];
 
-		/* Extract the attr */
-		tmp = color_char_to_attr(buf[4]);
+		/* If we have a longer string than expected ... */
+		if (buf[5])
+		{
+			/* Advance "buf" on by 4 */
+			buf += 4;
+
+			/* Extract the colour */
+			d_attr = color_text_to_attr(buf);
+		}
+		else
+		{
+			/* Extract the attr */
+			d_attr = color_char_to_attr(buf[4]);
+		}
 
 		/* Paranoia */
-		if (tmp < 0) return (PARSE_ERROR_GENERIC);
+		if (d_attr < 0) return (PARSE_ERROR_GENERIC);
 
 		/* Save the values */
-		r_ptr->d_attr = tmp;
-		r_ptr->d_char = sym;
+		r_ptr->d_attr = d_attr;
+		r_ptr->d_char = d_char;
 	}
 
 	/* Process 'I' for "Info" (one line only) */
@@ -1943,7 +2019,7 @@ errr parse_r_info(char *buf, header *head)
 		r_ptr->mexp = exp;
 	}
 
-	/* Process 'B' for "Blows" (up to four lines) */
+	/* Process 'B' for "Blows" */
 	else if (buf[0] == 'B')
 	{
 		int n1, n2;
@@ -1952,10 +2028,10 @@ errr parse_r_info(char *buf, header *head)
 		if (!r_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
 
 		/* Find the next empty blow slot (if any) */
-		for (i = 0; i < 4; i++) if (!r_ptr->blow[i].method) break;
+		for (i = 0; i < MONSTER_BLOW_MAX; i++) if (!r_ptr->blow[i].method) break;
 
 		/* Oops, no more slots */
-		if (i == 4) return (PARSE_ERROR_GENERIC);
+		if (i == MONSTER_BLOW_MAX) return (PARSE_ERROR_GENERIC);
 
 		/* Analyze the first field */
 		for (s = t = buf+2; *t && (*t != ':'); t++) /* loop */;
@@ -2058,7 +2134,7 @@ errr parse_r_info(char *buf, header *head)
 					return (PARSE_ERROR_INVALID_SPELL_FREQ);
 
 				/* Extract a "frequency" */
-				r_ptr->freq_spell = r_ptr->freq_inate = 100 / i;
+				r_ptr->freq_spell = r_ptr->freq_innate = 100 / i;
 
 				/* Start at next entry */
 				s = t;
@@ -2185,7 +2261,7 @@ errr parse_p_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		pr_ptr = &p_info[i];
+		pr_ptr = (player_race*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(pr_ptr->name = add_name(head, s)))
@@ -2197,7 +2273,7 @@ errr parse_p_info(char *buf, header *head)
 	{
 		int adj;
 
-		/* There better be a current pc_ptr */
+		/* There better be a current pr_ptr */
 		if (!pr_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
 
 		/* Start the string */
@@ -2445,7 +2521,7 @@ errr parse_c_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		pc_ptr = &c_info[i];
+		pc_ptr = (player_class*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(pc_ptr->name = add_name(head, s)))
@@ -2581,21 +2657,20 @@ errr parse_c_info(char *buf, header *head)
 	/* Process 'M' for "Magic Info" (one line only) */
 	else if (buf[0] == 'M')
 	{
-		int spell_book, spell_stat, spell_type, spell_first, spell_weight;
+		int spell_book, spell_stat, spell_first, spell_weight;
 
 		/* There better be a current pc_ptr */
 		if (!pc_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
 
 		/* Scan for the values */
-		if (5 != sscanf(buf+2, "%d:%d:%d:%d:%d",
-		                &spell_book, &spell_stat, &spell_type,
+		if (4 != sscanf(buf+2, "%d:%d:%d:%d",
+		                &spell_book, &spell_stat,
 		                &spell_first, &spell_weight))
 			return (PARSE_ERROR_GENERIC);
 
 		/* Save the values */
 		pc_ptr->spell_book = spell_book;
 		pc_ptr->spell_stat = spell_stat;
-		pc_ptr->spell_type = spell_type;
 		pc_ptr->spell_first = spell_first;
 		pc_ptr->spell_weight = spell_weight;
 	}
@@ -2754,7 +2829,7 @@ errr parse_h_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		h_ptr = &h_info[i];
+		h_ptr = (hist_type*)head->info_ptr + i;
 
 		/* Scan for the values */
 		if (4 != sscanf(buf, "N:%d:%d:%d:%d",
@@ -2852,7 +2927,7 @@ errr parse_b_info(char *buf, header *head)
 		error_idx = i;
 
 		/* Point at the "info" */
-		ot_ptr = &b_info[i];
+		ot_ptr = (owner_type*)head->info_ptr + i;
 
 		/* Store the name */
 		if (!(ot_ptr->owner_name = add_name(head, t)))
@@ -2932,7 +3007,7 @@ errr parse_g_info(char *buf, header *head)
 			error_idx = i;
 
 			/* Point at the "info" */
-			g_ptr = &g_info[i];
+			g_ptr = (byte*)head->info_ptr + i;
 
 			/* Find the colon before the subindex */
 			s = strchr(s, ':');
@@ -2961,6 +3036,120 @@ errr parse_g_info(char *buf, header *head)
 }
 
 
+/*
+ * Initialize the "flavor_info" array, by parsing an ascii "template" file
+ */
+errr parse_flavor_info(char *buf, header *head)
+{
+	int i;
+	
+	/* Current entry */
+	static flavor_type *flavor_ptr;
+
+
+	/* Process 'N' for "Number" */
+	if (buf[0] == 'N')
+	{
+		int tval, sval;
+		int result;
+
+		/* Scan the value */
+		result = sscanf(buf, "N:%d:%d:%d", &i, &tval, &sval);
+
+		/* Either two or three values */
+		if ((result != 2) && (result != 3)) return (PARSE_ERROR_GENERIC);
+
+		/* Verify information */
+		if (i <= error_idx) return (PARSE_ERROR_NON_SEQUENTIAL_RECORDS);
+
+		/* Verify information */
+		if (i >= head->info_num) return (PARSE_ERROR_TOO_MANY_ENTRIES);
+
+		/* Save the index */
+		error_idx = i;
+
+		/* Point at the "info" */
+		flavor_ptr = (flavor_type*)head->info_ptr + i;
+
+		/* Save the tval */
+		flavor_ptr->tval = (byte)tval;
+
+		/* Save the sval */
+		if (result == 2)
+		{
+			/* Megahack - unknown sval */
+			flavor_ptr->sval = SV_UNKNOWN;
+		}
+		else
+			flavor_ptr->sval = (byte)sval;
+	}
+
+	/* Process 'G' for "Graphics" */
+	else if (buf[0] == 'G')
+	{
+		char d_char;
+		int d_attr;
+
+		/* There better be a current flavor_ptr */
+		if (!flavor_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
+
+		/* Paranoia */
+		if (!buf[2]) return (PARSE_ERROR_GENERIC);
+		if (!buf[3]) return (PARSE_ERROR_GENERIC);
+		if (!buf[4]) return (PARSE_ERROR_GENERIC);
+
+		/* Extract d_char */
+		d_char = buf[2];
+
+		/* If we have a longer string than expected ... */
+		if (buf[5])
+		{
+			/* Advance "buf" on by 4 */
+			buf += 4;
+
+			/* Extract the colour */
+			d_attr = color_text_to_attr(buf);
+		}
+		else
+		{
+			/* Extract the attr */
+			d_attr = color_char_to_attr(buf[4]);
+		}
+
+		/* Paranoia */
+		if (d_attr < 0) return (PARSE_ERROR_GENERIC);
+
+		/* Save the values */
+		flavor_ptr->d_attr = d_attr;
+		flavor_ptr->d_char = d_char;
+	}
+
+	/* Process 'D' for "Description" */
+	else if (buf[0] == 'D')
+	{
+		/* There better be a current flavor_ptr */
+		if (!flavor_ptr) return (PARSE_ERROR_MISSING_RECORD_HEADER);
+
+		/* Paranoia */
+		if (!buf[1]) return (PARSE_ERROR_GENERIC);
+		if (!buf[2]) return (PARSE_ERROR_GENERIC);
+
+		/* Store the text */
+		if (!add_text(&flavor_ptr->text, head, buf + 2))
+			return (PARSE_ERROR_OUT_OF_MEMORY);
+	}
+
+	else
+	{
+		/* Oops */
+		return (PARSE_ERROR_UNDEFINED_DIRECTIVE);
+	}
+
+	/* Success */
+	return (0);
+}
+
+
 #else	/* ALLOW_TEMPLATES */
 
 #ifdef MACINTOSH
