@@ -111,9 +111,10 @@ struct maxima
 	u16b h_max;		/* Max size for "h_info[]" */
 	u16b b_max;		/* Max size per element of "b_info[]" */
 	u16b c_max;		/* Max size for "c_info[]" */
+	u16b flavor_max; /* Max size for "flavor_info[]" */
 
 	u16b o_max;		/* Max size for "o_list[]" */
-	u16b m_max;		/* Max size for "m_list[]" */
+	u16b m_max;		/* Max size for "mon_list[]" */
 };
 
 
@@ -187,7 +188,7 @@ struct object_kind
 	char x_char;		/* Desired object character */
 
 
-	byte flavor;		/* Special object flavor (or zero) */
+	u16b flavor;		/* Special object flavor (or zero) */
 
 
 	bool aware;			/* The player is "aware" of the item's effects */
@@ -250,28 +251,26 @@ struct ego_item_type
 	u32b name;			/* Name (offset) */
 	u32b text;			/* Text (offset) */
 
-	byte slot;			/* Standard slot value */
-	byte rating;		/* Rating boost */
-
-	byte level;			/* Minimum level */
-	byte rarity;		/* Object rarity */
-
-	byte tval[3];		/* Legal tval */
-	byte min_sval[3];	/* Minimum legal sval */
-	byte max_sval[3];	/* Maximum legal tval */
-	byte xtra;			/* Extra Sustain/Resist/Power */
-
-	byte max_to_h;		/* Maximum to-hit bonus */
-	byte max_to_d;		/* Maximum to-dam bonus */
-	byte max_to_a;		/* Maximum to-ac bonus */
-
-	byte max_pval;		/* Maximum pval */
-
 	s32b cost;			/* Ego-item "cost" */
 
 	u32b flags1;		/* Ego-Item Flags, set 1 */
 	u32b flags2;		/* Ego-Item Flags, set 2 */
 	u32b flags3;		/* Ego-Item Flags, set 3 */
+
+	byte level;			/* Minimum level */
+	byte rarity;		/* Object rarity */
+	byte rating;		/* Level rating boost */
+
+	byte tval[EGO_TVALS_MAX]; /* Legal tval */
+	byte min_sval[EGO_TVALS_MAX];	/* Minimum legal sval */
+	byte max_sval[EGO_TVALS_MAX];	/* Maximum legal sval */
+
+	byte max_to_h;		/* Maximum to-hit bonus */
+	byte max_to_d;		/* Maximum to-dam bonus */
+	byte max_to_a;		/* Maximum to-ac bonus */
+	byte max_pval;		/* Maximum pval */
+
+	byte xtra;			/* Extra sustain/resist/power */
 };
 
 
@@ -331,7 +330,7 @@ struct monster_race
 
 	s16b extra;				/* Unused (for now) */
 
-	byte freq_inate;		/* Inate spell frequency */
+	byte freq_innate;		/* Innate spell frequency */
 	byte freq_spell;		/* Other spell frequency */
 
 	u32b flags1;			/* Flags 1 (general) */
@@ -341,7 +340,7 @@ struct monster_race
 	u32b flags5;			/* Flags 5 (normal spells) */
 	u32b flags6;			/* Flags 6 (special spells) */
 
-	monster_blow blow[4];	/* Up to four blows per round */
+	monster_blow blow[MONSTER_BLOW_MAX]; /* Up to four blows per round */
 
 	byte level;				/* Level of creature */
 	byte rarity;			/* Rarity of creature */
@@ -368,32 +367,32 @@ struct monster_race
  */
 struct monster_lore
 {
-	s16b r_sights;			/* Count sightings of this monster */
-	s16b r_deaths;			/* Count deaths from this monster */
+	s16b sights;			/* Count sightings of this monster */
+	s16b deaths;			/* Count deaths from this monster */
 
-	s16b r_pkills;			/* Count monsters killed in this life */
-	s16b r_tkills;			/* Count monsters killed in all lives */
+	s16b pkills;			/* Count monsters killed in this life */
+	s16b tkills;			/* Count monsters killed in all lives */
 
-	byte r_wake;			/* Number of times woken up (?) */
-	byte r_ignore;			/* Number of times ignored (?) */
+	byte wake;				/* Number of times woken up (?) */
+	byte ignore;			/* Number of times ignored (?) */
 
-	byte r_xtra1;			/* Something (unused) */
-	byte r_xtra2;			/* Something (unused) */
+	byte xtra1;				/* Something (unused) */
+	byte xtra2;				/* Something (unused) */
 
-	byte r_drop_gold;		/* Max number of gold dropped at once */
-	byte r_drop_item;		/* Max number of item dropped at once */
+	byte drop_gold;			/* Max number of gold dropped at once */
+	byte drop_item;			/* Max number of item dropped at once */
 
-	byte r_cast_inate;		/* Max number of inate spells seen */
-	byte r_cast_spell;		/* Max number of other spells seen */
+	byte cast_innate;		/* Max number of innate spells seen */
+	byte cast_spell;		/* Max number of other spells seen */
 
-	byte r_blows[4];		/* Number of times each blow type was seen */
+	byte blows[MONSTER_BLOW_MAX]; /* Number of times each blow type was seen */
 
-	u32b r_flags1;			/* Observed racial flags */
-	u32b r_flags2;			/* Observed racial flags */
-	u32b r_flags3;			/* Observed racial flags */
-	u32b r_flags4;			/* Observed racial flags */
-	u32b r_flags5;			/* Observed racial flags */
-	u32b r_flags6;			/* Observed racial flags */
+	u32b flags1;			/* Observed racial flags */
+	u32b flags2;			/* Observed racial flags */
+	u32b flags3;			/* Observed racial flags */
+	u32b flags4;			/* Observed racial flags */
+	u32b flags5;			/* Observed racial flags */
+	u32b flags6;			/* Observed racial flags */
 };
 
 
@@ -483,8 +482,6 @@ struct object_type
 
 	u16b note;			/* Inscription index */
 
-	byte inscrip;		/* INSCRIP_XXX constant */
-
 	s16b next_o_idx;	/* Next object in stack (if any) */
 
 	s16b held_m_idx;	/* Monster holding us (if any) */
@@ -569,10 +566,6 @@ struct alloc_entry
  * the "quest level" is then the level past which progress is forbidden
  * until the quest is complete.  Note that the "QUESTOR" flag then could
  * become a more general "never out of depth" flag for monsters.
- *
- * Actually, in Angband 2.8.0 it will probably prove easier to restrict
- * the concept of quest monsters to specific unique monsters, and to
- * actually scan the dead unique list to see what quests are left.
  */
 struct quest
 {
@@ -623,23 +616,12 @@ struct store_type
 
 	s32b store_open;		/* Closed until this turn */
 
-	s16b table_num;			/* Table -- Number of entries */
-	s16b table_size;		/* Table -- Total Size of Array */
-	s16b *table;			/* Table -- Legal item kinds */
-
 	byte stock_num;			/* Stock -- Number of entries */
 	s16b stock_size;		/* Stock -- Total Size of Array */
 	object_type *stock;		/* Stock -- Actual stock items */
 };
 
 
-
-
-
-/*
- * The "name" of spell 'N' is stored as spell_names[X][N],
- * where X is 0 for mage-spells and 1 for priest-spells.
- */
 struct magic_type
 {
 	byte slevel;		/* Required level (to learn) */
@@ -771,7 +753,6 @@ struct player_class
 
 	byte spell_book;	/* Tval of spell books (if any) */
 	u16b spell_stat;	/* Stat for spells (if any) */
-	byte spell_type;	/* Spell type (mage/priest) */
 	u16b spell_first;	/* Level of first spell */
 	u16b spell_weight;	/* Weight that hurts spells */
 
@@ -906,19 +887,14 @@ struct player_type
 	byte confusing;		/* Glowing hands */
 	byte searching;		/* Currently searching */
 
-	u32b spell_learned1;	/* Spell flags */
-	u32b spell_learned2;	/* Spell flags */
-	u32b spell_worked1;		/* Spell flags */
-	u32b spell_worked2;		/* Spell flags */
-	u32b spell_forgotten1;	/* Spell flags */
-	u32b spell_forgotten2;	/* Spell flags */
+	byte spell_flags[PY_MAX_SPELLS]; /* Spell flags */
 
 	byte spell_order[PY_MAX_SPELLS];	/* Spell order */
 
 	s16b player_hp[PY_MAX_LEVEL];	/* HP Array */
 
 	char died_from[80];		/* Cause of death */
-	char history[4][60];	/* Initial history */
+	char history[250];	/* Initial history */
 
 	u16b total_winner;		/* Total winner */
 	u16b panic_save;		/* Panic save */
@@ -982,19 +958,6 @@ struct player_type
 
 	s16b new_spells;		/* Number of spells available */
 
-	s16b old_spells;
-
-	bool old_cumber_armor;
-	bool old_cumber_glove;
-	bool old_heavy_wield;
-	bool old_heavy_shoot;
-	bool old_icky_wield;
-
-	s16b old_lite;		/* Old radius of lite (if any) */
-	s16b old_view;		/* Old radius of view (if any) */
-
-	s16b old_food_aux;	/* Old value of food */
-
 	bool cumber_armor;	/* Mana draining armor */
 	bool cumber_glove;	/* Mana draining gloves */
 	bool heavy_wield;	/* Heavy weapon */
@@ -1140,3 +1103,20 @@ struct high_score
 
 	char how[32];		/* Method of death (string) */
 };
+
+
+typedef struct flavor_type flavor_type;
+
+struct flavor_type
+{
+	u32b text;      /* Text (offset) */
+	
+	byte tval;      /* Associated object type */
+	byte sval;      /* Associated object sub-type */
+
+	byte d_attr;    /* Default flavor attribute */
+	char d_char;    /* Default flavor character */
+
+	byte x_attr;    /* Desired flavor attribute */
+	char x_char;    /* Desired flavor character */
+};
