@@ -35,6 +35,15 @@ extern void (*core_aux)(cptr);
 
 /**** Available Functions ****/
 
+/* Case insensitive comparison between two strings */
+extern int my_stricmp(const char *s1, const char *s2);
+
+/* Copy a string */
+extern size_t my_strcpy(char *buf, const char *src, size_t bufsize);
+
+/* Concatenate two strings */
+extern size_t my_strcat(char *buf, const char *src, size_t bufsize);
+
 /* Test equality, prefix, suffix */
 extern bool streq(cptr s, cptr t);
 extern bool prefix(cptr s, cptr t);
